@@ -172,6 +172,8 @@ class Document(QWebPage):  # {{{
         self.enable_page_flip = self.page_flip_duration > 0.1
         self.font_magnification_step = opts.font_magnification_step
         self.wheel_flips_pages = opts.wheel_flips_pages
+        self.wheel_scroll_fraction = opts.wheel_scroll_fraction
+        self.line_scroll_fraction = opts.line_scroll_fraction
         self.tap_flips_pages = opts.tap_flips_pages
         self.line_scrolling_stops_on_pagebreaks = opts.line_scrolling_stops_on_pagebreaks
         screen_width = QApplication.desktop().screenGeometry().width()
@@ -258,7 +260,8 @@ class Document(QWebPage):  # {{{
         self.javascript('window.paged_display.read_document_margins()')
         self.set_bottom_padding(0)
         self.fit_images()
-        self.math_present = self.javascript('window.mathjax.check_for_math()', bool)
+        w = 1 if iswindows else 0
+        self.math_present = self.javascript('window.mathjax.check_for_math(%d)' % w, bool)
         self.init_hyphenate()
         self.javascript('full_screen.save_margins()')
         if self.in_fullscreen_mode:
@@ -549,6 +552,7 @@ class DocumentView(QWebView):  # {{{
     magnification_changed = pyqtSignal(object)
     DISABLED_BRUSH = QBrush(Qt.lightGray, Qt.Dense5Pattern)
     gesture_handler = lambda s, e: False
+    last_loaded_path = None
 
     def initialize_view(self, debug_javascript=False):
         self.setRenderHints(QPainter.Antialiasing|QPainter.TextAntialiasing|QPainter.SmoothPixmapTransform)
@@ -1239,7 +1243,8 @@ class DocumentView(QWebView):  # {{{
         painter.end()
 
     def wheelEvent(self, event):
-        if event.phase() != Qt.ScrollUpdate:
+        if event.phase() not in (Qt.ScrollUpdate, 0):
+            # 0 is Qt.NoScrollPhase which is not yet available in PyQt
             return
         mods = event.modifiers()
         num_degrees = event.angleDelta().y() // 8
@@ -1297,10 +1302,12 @@ class DocumentView(QWebView):  # {{{
         num_degrees_h = event.angleDelta().x() // 8
         vertical = abs(num_degrees) > abs(num_degrees_h)
         scroll_amount = ((num_degrees if vertical else num_degrees_h)/ 120.0) * .2 * -1 * 8
-        if vertical:
-            self.scroll_by(0, self.document.viewportSize().height() * scroll_amount)
-        else:
-            self.scroll_by(self.document.viewportSize().width() * scroll_amount, 0)
+        dim = self.document.viewportSize().height() if vertical else self.document.viewportSize().width()
+        amt =  dim * scroll_amount
+        mult = -1 if amt < 0 else 1
+        if self.document.wheel_scroll_fraction != 100:
+            amt = mult * max(1, abs(int(amt * self.document.wheel_scroll_fraction / 100.)))
+        self.scroll_by(0, amt) if vertical else self.scroll_by(amt, 0)
 
         if self.manager is not None:
             self.manager.scrolled(self.scroll_fraction)
@@ -1340,7 +1347,8 @@ class DocumentView(QWebView):  # {{{
                         self.document.at_bottom):
                     self.manager.next_document()
                 else:
-                    self.scroll_by(y=15)
+                    amt = int((self.document.line_scroll_fraction / 100.) * 15)
+                    self.scroll_by(y=amt)
         elif key == 'Up':
             if self.document.in_paged_mode:
                 self.paged_col_scroll(forward=False, scroll_past_end=not
@@ -1350,17 +1358,20 @@ class DocumentView(QWebView):  # {{{
                         self.document.at_top):
                     self.manager.previous_document()
                 else:
-                    self.scroll_by(y=-15)
+                    amt = int((self.document.line_scroll_fraction / 100.) * 15)
+                    self.scroll_by(y=-amt)
         elif key == 'Left':
             if self.document.in_paged_mode:
                 self.paged_col_scroll(forward=False)
             else:
-                self.scroll_by(x=-15)
+                amt = int((self.document.line_scroll_fraction / 100.) * 15)
+                self.scroll_by(x=-amt)
         elif key == 'Right':
             if self.document.in_paged_mode:
                 self.paged_col_scroll()
             else:
-                self.scroll_by(x=15)
+                amt = int((self.document.line_scroll_fraction / 100.) * 15)
+                self.scroll_by(x=amt)
         elif key == 'Back':
             if self.manager is not None:
                 self.manager.back(None)
